
class StandardArg:
    def __init__(self, line: str):
        self.flag = ""
        self.value = ""
        self.position = ""

class CollectionOfArgs:
    def __init__(self, std):
        pass

    def build_collection(self):
        pass



import re
import os
import subprocess
from commander.logger import *
from google.cloud import storage
from prompt_toolkit import prompt
from argparse import ArgumentParser
from typing import Tuple
from commander.constants import OPTIONS, PLATFORM
from commander.walker.datacache import cache_data, load_cache_data
from prompt_toolkit.completion import Completer, Completion, ThreadedCompleter

class SideCompletion(Completer):
    def __init__(self):
        self.options = []

    def get_completions(self, document, complete_event):
        # Provide custom autocomplete suggestions based on the current input
        word_before_cursor = document.get_word_before_cursor(WORD=True)
        if word_before_cursor:
            try:
                if(word_before_cursor.startswith('gs://')):
                    # TODO self.options = gcp.listdir
                    pass
                else:
                    self.options = os.listdir(word_before_cursor)
            except:
                pass

            last_word = word_before_cursor.split('/')[-1:][0]


            for opt in self.options:
                # if last_word in opt:
                if opt.startswith(last_word):
                    rest_of_word = opt[len(last_word):]
                    yield Completion(rest_of_word)

def echo_localdir(args: ArgumentParser, action: str,base_path_to_fzf: str, fzf: subprocess.Popen)->None:
    for dirpath, dirnames, filenames in os.walk(base_path_to_fzf):
        dirpath = dirpath.replace('\\', '/')
        for filename in filenames:
            line = ""
            if action.startswith("ls"):
                line = f"{dirpath}/\n"
            else:
                line = f"{dirpath}/{filename}\n"
            fzf.stdin.write((line).encode())

def echo_gcpdir(args: ArgumentParser, action: str, bucket_name: str, fzf: subprocess.Popen)->None:
    client = storage.Client()
    bucket_contents = client.list_blobs(bucket_name)
    for i in bucket_contents:
        fzf.stdin.write((i).encode())


def walkdir(args: ArgumentParser, action: str, base_path_to_fzf: str)->str:
    base_path_to_fzf = base_path_to_fzf.replace('\n', '')
    fzf = subprocess.Popen(["fzf"], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    try:
        if base_path_to_fzf.startswith("gs://"):
            echo_gcpdir(args, action, base_path_to_fzf, fzf)
        else:
            echo_localdir(args, action, base_path_to_fzf, fzf)
    except:
        pass

    output, _ = fzf.communicate()
    output = output.decode()
    return output

def start_fzf(args: ArgumentParser, to_send: str)->str:
    fzf = subprocess.Popen(["fzf"], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
    output, _ = fzf.communicate(to_send.encode())
    output = output.decode()

    if args.debug:
        logging.info("Selected item: ", output)
    return output

def ls_logic(args: ArgumentParser, action:str, base_path_to_fzf: str)->Tuple[str, str]:
        left_selected: str = walkdir(args, action, base_path_to_fzf) 
        right_selected: str = ""
        return left_selected, right_selected

def mv_logic(args: ArgumentParser, action:str, base_path_to_fzf: str)->Tuple[str, str]:
        left_selected: str = walkdir(args, action, base_path_to_fzf) 
        basepath_options = load_cache_data(args, fname="basepaths")
        right_selected: str = start_fzf(args, basepath_options)

        side_winder_completer = SideCompletion()
        thread_completer = ThreadedCompleter(side_winder_completer)
        if args.vim == True:
            right_selected = prompt("Move to: ", default=right_selected.replace('\n', ''), completer=thread_completer, vi_mode=True)
        else: 
            right_selected = prompt("Move to: ", default=right_selected.replace('\n', ''), completer=thread_completer,)
        return left_selected, right_selected

def cp_logic(args: ArgumentParser, action:str, base_path_to_fzf: str)->Tuple[str, str]:
        left_selected: str = walkdir(args, action, base_path_to_fzf) # TODO add OPEN in browser option
        basepath_options = load_cache_data(args, fname="basepaths")
        right_selected: str = start_fzf(args, basepath_options)
        if args.vim == True:
            right_selected = prompt("Copy to: ", default=right_selected.replace('\n', ''), vi_mode=True)
        else: 
            right_selected = prompt("Copy to: ", default=right_selected.replace('\n', ''))
        return left_selected, right_selected

def rm_logic(args: ArgumentParser, action:str, base_path_to_fzf: str)->Tuple[str, str]:
        left_selected: str = walkdir(args, action, base_path_to_fzf) # TODO add OPEN in browser option
        right_selected: str = ""
        return left_selected, right_selected

def action_logic(args: ArgumentParser, action: str, base_path_to_fzf: str)->str:
    left_selected = ""
    right_selected = ""
    if action.startswith("ls"):
        left_selected, right_selected = ls_logic(args, action, base_path_to_fzf)
    elif action.startswith("mv"):
        left_selected, right_selected = mv_logic(args, action, base_path_to_fzf)
    elif action.startswith("cp"):
        left_selected, right_selected = cp_logic(args, action, base_path_to_fzf)
    elif action.startswith("rm"):
        left_selected, right_selected = rm_logic(args, action, base_path_to_fzf)
    else:
        print(f"Invalid movement option: {action}")

    cmd = ""
    if base_path_to_fzf.startswith('gs://'):
        cmd = f"gsutil -m {action} {left_selected} {right_selected}".replace('\n', ' ')
    else:
        cmd = f"{action} {left_selected} {right_selected}".replace('\n', ' ')

    return cmd

def filter_basepath(args: ArgumentParser, basepath_regex_str: str):
    basepath_regex = re.compile(basepath_regex_str)

    basepath_options = load_cache_data(args, fname="basepaths")
    filtered_basepath_options = ""
    for basepath in basepath_options.split('\n'):
        match = re.search(basepath_regex, basepath)
        if not match:
            filtered_basepath_options += basepath+'\n'
    
    cache_data(args, str_to_cache=filtered_basepath_options, fname="basepaths", filemode="w")


def surf(args: ArgumentParser, basepath: str):
    """Start sidewinder, event_loop for the TUI application.
    """
    if args.basepath != "":
        if args.filter_basepath != None:
            filter_basepath(args, args.filter_basepath)
        cache_data(args, str_to_cache=args.basepath, fname="basepaths")
        basepath_options = load_cache_data(args, fname="basepaths")
        action: str = start_fzf(args, OPTIONS)
        base_path_to_fzf = start_fzf(args, basepath_options)
        print("Base path to look inside of ",base_path_to_fzf)

        if(base_path_to_fzf.startswith("gs://")):
            google_env_variable = 'GOOGLE_APPLICATION_CREDENTIALS'
            try: 
                os.environ[google_env_variable] 
            except: 
                print(f"gsutil credentials environment variable not set. \nCheck the {google_env_variable} environment variable.")


        result_cmd = action_logic(args, action, base_path_to_fzf)

        side_winder_completer = SideCompletion()
        thread_completer = ThreadedCompleter(side_winder_completer)
        if args.vim == True:
            result_cmd = prompt("Run this command? ", default=result_cmd, completer=thread_completer,vi_mode=True)
        else: 
            result_cmd = prompt("Run this command? ", default=result_cmd, completer=thread_completer)

        if args.debug == True:
            print("would execute: ", result_cmd)
        else:
            if result_cmd != "":
                command_exec = ""
                if PLATFORM.startswith("win"):
                    command_exec = subprocess.run(["powershell.exe", "-Command", result_cmd],
                                            stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout.decode()
                else:
                    command_exec = subprocess.run([result_cmd],
                                            stdout=subprocess.PIPE, stderr=subprocess.PIPE).stdout.decode()

                print(command_exec)


import os
import json
import argparse 
from commander.logger import *
from commander.args import arg_parse_rules
from argparse import ArgumentParser
from commander.tui import entry_point, collect_pipe_input
from commander.constants import CWD
from typing import Tuple

def get_argument_setting_values(arg_settings: dict)->Tuple[list, dict]:
    """This function takes the settings for any arg in the args/json/*.json file and parses them.

    Args:
        arg_settings (dict): The collection of settings under any arg.

    Returns:
        tuple[list, dict]: list, is the positional parameters and dict is the kwargs to unpack to argparser.parse_args() function.
    """
    positional_args = [arg_settings["flags"]["small_form"], arg_settings["flags"]["long_form"]]
    kwargs = {"help": arg_settings["help"], "required": arg_settings["required"]}

    kwargs = arg_parse_rules.should_type_be_added(arg_settings, kwargs)
    kwargs = arg_parse_rules.should_action_be_added(arg_settings, kwargs)
    kwargs = arg_parse_rules.should_choice_be_added(arg_settings, kwargs)
    kwargs = arg_parse_rules.should_default_be_added(arg_settings, kwargs)

    return positional_args, kwargs

def build_args_and_parse(arg_json_directory: str)->ArgumentParser:
    parser: argparse.Argu = argparse.ArgumentParser(description="My application description")

    for filename in os.listdir(arg_json_directory):
        filepath = os.path.join(arg_json_directory, filename)
        with open(filepath) as json_file:
            data: dict = json.load(json_file)
            for arg_name, arg in data.items():
                positional_parameters, kwparameters = get_argument_setting_values(arg_settings=arg)

                try:
                    parser.add_argument(*positional_parameters, **kwparameters)
                except Exception as e:
                    print(f"There was a problem processing the values in {filepath} for argument '{arg_name}'")
                    logging.warn(f"There was a problem processing the values in {filepath} for argument '{arg_name}'")
                    logging.warn("Full params: " + arg)
                    logging.warn("Filtered params\n"+ positional_parameters + "\n" + kwparameters)
                    logging.warn("Trace:"+ e)

    args = parser.parse_args()

    if args.debug == True:
        logging.info("Created Arguments:", args)
    return args

def main():
    try:
        arg_json_directory = f"{CWD}/args/json/"

        args: ArgumentParser = build_args_and_parse(arg_json_directory)

        # entry_point(collect_pipe_input(), args)
        entry_point("", args)
    except KeyboardInterrupt as e:
        print("Process closed due to keyboard interrupt.")

if __name__ == "__main__":
    main()


import sys
from commander.logger import *
from commander.walker import sidewinder
from argparse import ArgumentParser

# TODO make this function non-blocking if there is no std input, then uncomment in main.
def collect_pipe_input()->str:
    """Collect platform specific standard input text, i.e. from | 

    Returns:
        str: Mutliline str of data from pipe.
    """
    std_in_text = ""
    for line in sys.stdin:
        std_in_text += line
    return std_in_text


def entry_point(std_in_text: str, args: ArgumentParser):
    """Commandline args to pass to the program.
    Examples: --vi enables vim keybinds.
    Or standard input from | char

    Args:
        args (List[str]): program arguments to enable certain settings.
    """
    if std_in_text != "":
        basepath = sidewinder.start_fzf(args, std_in_text)
        sidewinder.surf(args, basepath=basepath)
    else:
        if args.debug:
            logging.info("Starting sidewinder w/ no std input")
        sidewinder.surf(args, "")
        



from commander.logger import *
from argparse import ArgumentParser
from commander.constants import CACHE_DIR


def cache_data(args: ArgumentParser, str_to_cache: str, fname: str, filemode: str = 'a'):
    """Saves data to the cache locaton

    Args:
        args (ArgumentParser): Command line args for the program.
    """
    filename = f"{CACHE_DIR}{fname}"
    not_duplicate = filter_duplicates(args, item_to_find=str_to_cache, filename=filename)
    if not_duplicate:
        if args.debug == True:
            logging.info(f"Opening cache file at {filename}")
        with open(filename, filemode) as f:
            f.write(str_to_cache + '\n')

def load_cache_data(args: ArgumentParser, fname: str)->str:
    filename = f"{CACHE_DIR}{fname}"
    try:
        with open(filename, 'r') as f:
            return f.read()
    except (FileNotFoundError, EOFError):
        if args.debug == True:
            logging.warn(f"Was unable to open cache file {filename} verify if it should've been created.")
        return None

def filter_duplicates(args: ArgumentParser, item_to_find, filename: str)->bool:
    """Used to check if item is already in cache file.
    Returns:
        bool: Returns True if item not in cache file(new item), False otherwise
    """
    cache_file_contents = load_cache_data(args, filename.split('/')[-1:][0])
    if cache_file_contents == None:
        return True

    if item_to_find+'\n' not in cache_file_contents:
        if args.debug == True:
            logging.info(f"Found new basepath {item_to_find} to add to {filename}")
        return True

    return False


from commander.args.arg_parse_rules import *

# should_default_be_added(settings: dict, arg_parse_params: dict)->dict:

# arg_settings = {'required': False, 'flags': {'small_form': '-b', 'long_form': '--basepath'}, 'action': '', 'help': 'A flag to predefine basepath', 'type': 'int', 'default': '', 'nargs': '+', 'choices': []}
# kwargs = {"help": arg_settings["help"], "required": arg_settings["required"]}

# NOTE to-self, dicts are passed by reference, so if you modify a dict in a function then it will modify it outside the function.

def test_should_default_be_added():
    kwargs = {}
    arg_settings = {'action': '', 'default': 'test'}
    assert should_default_be_added(arg_settings, kwargs.copy()) == {'default': 'test'}
    arg_settings = {'action': '', 'default': 'default behavior here'}
    assert should_default_be_added(arg_settings, kwargs.copy()) == {'default': 'default behavior here'}


def test_should_action_be_added():
    kwargs = {}
    arg_settings = {'action': 'store_true'}
    assert should_action_be_added(arg_settings, kwargs.copy()) == {'action': 'store_true'}
    arg_settings['action'] = 'test'
    assert should_action_be_added(arg_settings, kwargs.copy()) == {'action': 'test'}


def test_should_type_be_added():
    kwargs = {}
    arg_settings = {'action': 'test', 'type': 'int'}
    assert should_type_be_added(arg_settings, kwargs.copy()) == {'type': int}
    arg_settings['type'] = 'str'
    assert should_type_be_added(arg_settings, kwargs.copy()) == {'type': str}

    arg_settings = {'action': '', 'type': 'int'}
    assert should_type_be_added(arg_settings, kwargs.copy()) == {}
    arg_settings['type'] = 'str'
    assert should_type_be_added(arg_settings, kwargs.copy()) == {}

    arg_settings['type'] = ''
    assert should_type_be_added(arg_settings, kwargs.copy()) == {}

    arg_settings['action'] = 'x'
    assert should_type_be_added(arg_settings, kwargs.copy()) == {}


def test_should_choice_be_added():
    kwargs = {}
    arg_settings = {'action': '', 'choices': ['choice1', 'choice2']}
    print(kwargs)
    assert should_choice_be_added(arg_settings, kwargs.copy()) == {'choices': ['choice1', 'choice2']}
    print(kwargs)
    arg_settings = {'action': '', 'choices': ['sample2']}
    assert should_choice_be_added(arg_settings, kwargs.copy()) == {'choices': ['sample2']}
    print(kwargs)

    arg_settings = {'action': '', 'choices': []}
    assert should_choice_be_added(arg_settings, kwargs.copy()) == {}
    arg_settings = {'action': 'x', 'choices': []}
    assert should_choice_be_added(arg_settings, kwargs.copy()) == {}



"""
This file contains filtering rules for which options should be passed to the argparser.parse_args() function.
It takes the .json value of an argument from args/json/*.json and filters it to decide if it should add an option to the final dict{}
"""

def should_default_be_added(settings: dict, arg_parse_params: dict)->dict:
    if settings['action'] not in ["store_true"] and settings['default'] != '':
        arg_parse_params.update({"default": settings["default"]})
    return arg_parse_params

def should_action_be_added(settings: dict, arg_parse_params: dict)->dict:
    if settings['action'] != '':
        arg_parse_params.update({"action": settings['action']})
    return arg_parse_params

def should_type_be_added(settings: dict, arg_parse_params: dict)->dict:
    if settings['action'] != '' and settings['type'] != '':
        if settings['type'] == 'int':
            arg_parse_params.update({"type": int})
        elif settings['type'] == 'str':
            arg_parse_params.update({"type": str})
    return arg_parse_params

def should_choice_be_added(settings: dict, arg_parse_params: dict)->dict:
    if settings['choices'] != [] and settings['action'] == '':
        arg_parse_params.update({"choices": settings['choices']})
    return arg_parse_params


import logging

logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s %(levelname)s %(message)s',
                    filename='gcloud-commander.log',
                    filemode='w')


import os
import platform

CWD = os.path.dirname(os.path.abspath(__file__)).replace('\\', '/')
CACHE_DIR = f"{CWD}/cache/"


PLATFORM = platform.system().lower()


OPTIONS = """
ls
cp
mv
rm
""".strip()


from setuptools import setup, find_packages
from pathlib import Path


this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name='gcloud-commander',
    version='0.11',
    packages=find_packages(),
    python_requires='>=3.5',
    install_requires=[
        'prompt_toolkit>=3.0.36',
        'protobuf>=4.21.12',
        'python_editor>=1.0.4',
        'setuptools>=63.2.0',
        'google-cloud-storage'
    ],
    entry_points={
        'console_scripts': [
            'gcmdr=commander.main:main'
        ]
    },
    author='Garrett Jones',
    author_email='jonesgc137@gmail.com',
    description='FZF based filemanager for Google Cloud',
    url='https://github.com/Lonestar137/gcloud-commander',
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
    include_package_data=True,
    package_data={'': ["*json/*", "*cache/sample"]}
)
